"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const react = require("react");
const reactStore = require("@tanstack/react-store");
const lodashEs = require("lodash-es");
const utils = require("./utils.cjs");
const encoding = require("./encoding.cjs");
const useApi = require("./use-api.cjs");
const api = require("./api.cjs");
let isInitialized = false;
const createParams = () => {
  let paramsStore = null;
  let api$1 = null;
  const getLatestParams = (api2) => {
    const params = new URLSearchParams(api2.getSearch());
    const paramsMap = {};
    for (const [key, value] of params) {
      paramsMap[key] = value;
    }
    return paramsMap;
  };
  const init = (contextApi, isClient2) => {
    if (isClient2 && isInitialized) {
      return;
    }
    isInitialized = true;
    console.log("INIT");
    console.log("contextApi", contextApi);
    api$1 = api.withBatch(contextApi ?? (isClient2 ? api.defaultApi() : api.dummyApi));
    if (isClient2) {
      api$1.registerListener((state) => {
        if (utils.isParamsTransition(state)) {
          return;
        }
        paramsStore.setState(() => getLatestParams(api$1));
      });
      paramsStore = new reactStore.Store(getLatestParams(api$1));
    }
  };
  const decodeWithDefault = (value, defaultValue, options) => {
    var _a, _b;
    const internalDefaultValue = utils.getValue(defaultValue);
    try {
      if (value === void 0) {
        return internalDefaultValue;
      }
      const decoded = encoding.decodeParam(value, options.decode);
      const isValid = options.validate ? options.validate(decoded) : true;
      if (!isValid) {
        (_a = options.onError) == null ? void 0 : _a.call(options, value, decoded);
        return internalDefaultValue;
      }
      return decoded;
    } catch {
      const onErrorValue = (_b = options.onError) == null ? void 0 : _b.call(options, value, void 0);
      return onErrorValue ?? internalDefaultValue;
    }
  };
  const useParamGet = (paramName, options) => {
    const contextApi = useApi.useContextApi();
    console.log("USE PARAM GET");
    init(contextApi, utils.isClient);
    const { defaultValue } = options;
    const smartDefaultValue = utils.useSmartValue(defaultValue);
    const value = utils.isClient ? reactStore.useStore(paramsStore, (s) => s[paramName]) : getLatestParams(api$1)[paramName];
    const decodedValue = react.useMemo(() => decodeWithDefault(value, smartDefaultValue, options), [value, smartDefaultValue, options]);
    return utils.useSmartValue(decodedValue);
  };
  const useParamSet = (paramName, options) => {
    const contextApi = useApi.useContextApi();
    init(contextApi, utils.isClient);
    const { defaultValue } = options;
    const smartDefaultValue = utils.useSmartValue(defaultValue);
    return react.useCallback((value, state) => {
      if (!utils.isClient) {
        throw new Error("Cannot use react params on the server");
      }
      const { updateType = "replaceIn", encode } = options;
      const currentValue = paramsStore.state[paramName];
      const internalValue = typeof value === "function" ? value(decodeWithDefault(currentValue, smartDefaultValue, options)) : value;
      const newValue = encoding.encodeParam(internalValue, encode);
      if (currentValue === newValue || currentValue === void 0 && lodashEs.isEqual(smartDefaultValue, value)) {
        return;
      }
      paramsStore.setState((s) => {
        return { ...s, [paramName]: newValue };
      });
      const search = api$1.getSearch();
      const clearAll = !updateType.endsWith("In");
      const isReplace = updateType.startsWith("replace");
      const searchParams = !clearAll ? new URLSearchParams(search) : new URLSearchParams();
      if (!lodashEs.isEqual(smartDefaultValue, value)) {
        searchParams.set(paramName, newValue);
      } else {
        searchParams.delete(paramName);
      }
      const newHref = `?${searchParams.toString()}`;
      const internalState = { [utils.paramsTransitioning]: true, ...state };
      if (isReplace) {
        api$1.replaceState(newHref, internalState);
      } else {
        api$1.pushState(newHref, internalState);
      }
    }, [paramName, smartDefaultValue, options]);
  };
  const useParam = (paramName, options) => {
    const value = useParamGet(paramName, options);
    const setter = useParamSet(paramName, options);
    return [value, setter];
  };
  return {
    useParamGet,
    useParamSet,
    useParam,
    getInternals: () => {
      return {
        paramsStore,
        api: api$1
      };
    }
  };
};
exports.createParams = createParams;
//# sourceMappingURL=core.cjs.map
