"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const react = require("react");
const lodashEs = require("lodash-es");
const getRenderingType = () => {
  const isBrowser = typeof window != "undefined" && window.document;
  return isBrowser ? "client" : "server";
};
const getValue = (value) => {
  if (typeof value === "function") {
    return value();
  }
  return value;
};
const isClient = getRenderingType() === "client";
const isServer = getRenderingType() === "server";
const paramsTransitioning = "__ParamsTransitioning";
const isParamsTransition = (state) => state != null && typeof state === "object" && paramsTransitioning in state;
const useSmartValue = (value) => {
  const internalValue = getValue(value);
  const ref = react.useRef(void 0);
  if (ref.current === void 0) {
    ref.current = internalValue;
  }
  if (lodashEs.isEqual(ref.current, internalValue)) {
    return ref.current;
  } else {
    ref.current = internalValue;
    return internalValue;
  }
};
const useMemoOptions = (options, options2, options3) => {
  const smartOptions1 = useSmartValue(options);
  const smartOptions2 = useSmartValue(options2);
  const smartOptions3 = useSmartValue(options3);
  return react.useMemo(() => mergeOptions(smartOptions1, smartOptions2, smartOptions3), [smartOptions1, smartOptions2, smartOptions3]);
};
const mergeOptions = (...options) => {
  return Object.assign({}, ...options);
};
const camelCaseToKebab = (str) => str.replace(/[A-Z]+(?![a-z])|[A-Z]/g, ($, ofs) => (ofs ? "-" : "") + $.toLowerCase());
exports.camelCaseToKebab = camelCaseToKebab;
exports.getRenderingType = getRenderingType;
exports.getValue = getValue;
exports.isClient = isClient;
exports.isParamsTransition = isParamsTransition;
exports.isServer = isServer;
exports.mergeOptions = mergeOptions;
exports.paramsTransitioning = paramsTransitioning;
exports.useMemoOptions = useMemoOptions;
exports.useSmartValue = useSmartValue;
//# sourceMappingURL=utils.cjs.map
