import { createParams } from "./core.js";
import { camelCaseToKebab, getValue, useMemoOptions } from "./utils.js";
import { encodeParam, createListSerializer, dateSerializer, datetimeSerializer, booleanSerializer, numberSerializer, stringSerializer, defaultDecoder, defaultEncoder } from "./encoding.js";
import { once } from "lodash-es";
import { useMemo } from "react";
import { batch } from "@tanstack/react-store";
const typeBasedOptions = {
  string: {
    ...stringSerializer
  },
  number: {
    ...numberSerializer
  },
  boolean: {
    ...booleanSerializer
  },
  datetime: {
    ...datetimeSerializer
  },
  date: {
    ...dateSerializer
  }
};
class InternalBuilder {
  constructor(state) {
    this.state = state;
    this.transformParams = null;
  }
  withSerializer(serializer) {
    this.state = {
      ...this.state,
      ...serializer
    };
    return this;
  }
  validate(validator) {
    this.state = {
      ...this.state,
      ...validator
    };
    return this;
  }
  withDefault(value) {
    this.state = {
      ...this.state,
      defaultValue: value
    };
    return this;
  }
  transform(params2) {
    this.transformParams = params2;
    return this;
  }
}
const createInternalBuilder = (key, value) => {
  const options = typeBasedOptions[key] || {};
  const listOptions = key === "list" ? createListSerializer(value) : {};
  const defaultSerializers = {
    encode: defaultEncoder,
    decode: defaultDecoder
  };
  const state = {
    onError: () => {
    },
    validate: () => true,
    defaultValue: null,
    updateType: "replaceIn",
    ...defaultSerializers,
    ...options,
    ...value || {},
    ...listOptions
  };
  return new InternalBuilder(state);
};
const p = new Proxy({}, {
  get(_target, prop) {
    return (value) => {
      const key = prop;
      return createInternalBuilder(key, value);
    };
  }
});
const defaultSetTransformer = {
  set: ({ set: setValue }) => {
    return setValue;
  }
};
const getWithPrefix = (prefix, prop) => {
  return prefix ? `${prefix}-${prop}` : prop;
};
const buildSingle = ({ builder, params: params2, prop, globalOptions }) => {
  const options = builder.state;
  const transformParams = builder.transformParams;
  const typedOptions = options;
  const { set } = transformParams || defaultSetTransformer;
  return {
    useSet: (options2) => {
      const memoedOptions = useMemoOptions(typedOptions, globalOptions, options2);
      const paramName = getWithPrefix(memoedOptions.prefix, prop);
      const setter = params2.useParamSet(paramName, memoedOptions);
      return useMemo(() => {
        return set({
          set: setter
        });
      }, [setter]);
    },
    use: (options2) => {
      const memoedOptions = useMemoOptions(typedOptions, globalOptions, options2);
      const paramName = getWithPrefix(memoedOptions.prefix, prop);
      console.log("bla bla bls");
      const res = params2.useParamGet(paramName, memoedOptions);
      const setter = params2.useParamSet(paramName, memoedOptions);
      const setterTransformed = useMemo(() => {
        return set({
          set: setter
        });
      }, [setter]);
      return [
        res,
        setterTransformed
      ];
    }
  };
};
let params = null;
const init = once(() => {
  params = createParams();
});
function create(schema, globalOptions) {
  init();
  let internalGlobalOptions = globalOptions;
  const proxy = new Proxy({}, {
    get(_target, prop) {
      if (prop === "withOptions") {
        return (globalOptions2) => {
          internalGlobalOptions = {
            ...internalGlobalOptions,
            ...globalOptions2
          };
          return proxy;
        };
      }
      if (prop === "build") {
        return (values) => {
          const params2 = Object.entries(values).map(([key, value]) => {
            const schemaKey = camelCaseToKebab(key);
            const builder2 = schema[schemaKey];
            const { defaultValue, encode } = builder2.state;
            if (value === getValue(defaultValue)) {
              return void 0;
            }
            const encodedValue = encodeParam(value, encode);
            return `${key}=${encodedValue}`;
          }).filter(Boolean);
          return params2.join("&");
        };
      }
      if (prop === "batch") {
        return (fn) => {
          const { api } = params.getInternals();
          batch(() => {
            api.batch(fn);
          });
        };
      }
      const schemaField = camelCaseToKebab(prop);
      const builder = schema[schemaField];
      return buildSingle({
        builder,
        params,
        prop,
        globalOptions: internalGlobalOptions
      });
    }
  });
  return proxy;
}
function createSingle(prop, definition) {
  init();
  const builder = definition;
  return buildSingle({
    builder,
    params,
    prop
  });
}
export {
  create,
  createSingle,
  p
};
//# sourceMappingURL=params.js.map
